#INCLUDE "PROTHEUS.CH"
#INCLUDE "FWMVCDEF.CH"

//-------------------------------------------------------------------
/*/{Protheus.doc} COMP022_MVC
Exemplo de montagem da modelo e interface para uma estrutura
pai/filho em MVC com estrutura de calculo, interceptacao da
gravacao dos dados (COMMIT) e folders

@author Ernani Forastieri e Rodrigo Antonio Godinho
@since 05/10/2009
@version P10
/*/
//-------------------------------------------------------------------
User Function COMP022_MVC()
Local oBrowse

oBrowse := FWmBrowse():New()
oBrowse:SetAlias( 'ZA1' )
oBrowse:SetDescription( 'Musicas' )
oBrowse:Activate()

Return( NIL )


//-------------------------------------------------------------------
Static Function MenuDef()
Local aRotina := {}
/*
ADD OPTION aRotina Title 'Pesquisar'  Action 'PesqBrw'             OPERATION 1 ACCESS 0
ADD OPTION aRotina Title 'Visualizar' Action 'VIEWDEF.COMP022_MVC' OPERATION 2 ACCESS 0
ADD OPTION aRotina Title 'Incluir'    Action 'VIEWDEF.COMP022_MVC' OPERATION 3 ACCESS 0
ADD OPTION aRotina Title 'Alterar'    Action 'VIEWDEF.COMP022_MVC' OPERATION 4 ACCESS 0
ADD OPTION aRotina Title 'Excluir'    Action 'VIEWDEF.COMP022_MVC' OPERATION 5 ACCESS 0
ADD OPTION aRotina Title 'Imprimir'   Action 'VIEWDEF.COMP022_MVC' OPERATION 8 ACCESS 0
*/
aAdd( aRotina, { 'Pesquisar' , 'PesqBrw'            , 0, 1, 0, .T. } )
aAdd( aRotina, { 'Visualizar', 'VIEWDEF.COMP022_MVC', 0, 2, 0, NIL } )
aAdd( aRotina, { 'Incluir'   , 'VIEWDEF.COMP022_MVC', 0, 3, 0, NIL } )
aAdd( aRotina, { 'Alterar'   , 'VIEWDEF.COMP022_MVC', 0, 4, 0, NIL } )
aAdd( aRotina, { 'Excluir'   , 'VIEWDEF.COMP022_MVC', 0, 5, 0, NIL } )
aAdd( aRotina, { 'Imprimir'  , 'VIEWDEF.COMP022_MVC', 0, 8, 0, NIL } )
aAdd( aRotina, { 'Copiar'    , 'VIEWDEF.COMP022_MVC', 0, 9, 0, NIL } )

Return aRotina


//-------------------------------------------------------------------
Static Function ModelDef()
// Cria a estrutura a ser usada no Modelo de Dados
Local oStruZA1 := FWFormStruct( 1, 'ZA1', /*bAvalCampo*/, /*lViewUsado*/ )
Local oStruZA2 := FWFormStruct( 1, 'ZA2', /*bAvalCampo*/, /*lViewUsado*/ )
Local oModel

// Cria o objeto do Modelo de Dados
oModel := MPFormModel():New( 'COMP022M', /*bPreValidacao*/, /*bPosValidacao*/, /*bCommit*/, /*bCancel*/ )
//oModel := MPFormModel():New( 'COMP022M', /*bPreValidacao*/, /*bPosValidacao*/, { |oMdl| COMP022CM2( oMdl ) }, /*bCancel*/ )

// Adiciona ao modelo uma estrutura de formulrio de edio por campo
oModel:AddFields( 'ZA1MASTER', NIL, oStruZA1 )

// Adiciona ao modelo uma estrutura de formulrio de edio por grid
oModel:AddGrid( 'ZA2DETAIL', 'ZA1MASTER', oStruZA2, /*bLinePre*/, /*bLinePost*/, /*bPreVal*/, /*bPosVal*/, /*BLoad*/ )

// Adiciona ao modelo uma estrutura de formulrio de campos calculados
// AddCalc(cId, cOwner , cIdForm , cIdField , cIdCalc, cOperation, bCond
oModel:AddCalc( 'COMP022CALC1', 'ZA1MASTER', 'ZA2DETAIL', 'ZA2_AUTOR', 'ZA2__TOT01', 'COUNT', { | oFW | COMP022CAL( oFW, .f. ) } )


// Faz relaciomaneto entre os compomentes do model
oModel:SetRelation( 'ZA2DETAIL', { { 'ZA2_FILIAL', 'xFilial( "ZA2" ) ' } , { 'ZA2_MUSICA', 'ZA1_MUSICA' } } , 'ZA2_FILIAL + ZA2_MUSICA' )

// Liga o controle de nao repeticao de linha
oModel:GetModel( 'ZA2DETAIL' ):SetUniqueLine( { 'ZA2_AUTOR' } )

// Adiciona a descricao do Modelo de Dados
oModel:SetDescription( 'Modelo de Musicas' )

// Adiciona a descricao do Componente do Modelo de Dados
oModel:GetModel( 'ZA1MASTER' ):SetDescription( 'Dados da Musica' )
oModel:GetModel( 'ZA2DETAIL' ):SetDescription( 'Dados do Autor'  )


Return oModel


//-------------------------------------------------------------------
Static Function ViewDef()
// Cria a estrutura a ser usada na View
Local oStruZA1 := FWFormStruct( 2, 'ZA1' )
Local oStruZA2 := FWFormStruct( 2, 'ZA2' )
 // Cria um objeto de Modelo de Dados baseado no ModelDef do fonte informado
Local oModel   := FWLoadModel( 'COMP022_MVC' )
Local oView
Local oCalc1

// Cria o objeto de Estrutura
oCalc1 := FWFormViewStruct():New()
oCalc1:AddField( 'ZA2__TOT01', '01', 'Qtd.Aut.Impar', 'Qtd.Aut.Impar'  , GetHlpSoluc( 'ZA2_AUTOR' ) , 'N', '9999', Nil, '', .T., '', '', {} , 0, '' )

// Cria o objeto de View
oView := FWFormView():New()

// Define qual o Modelo de dados ser utilizado
oView:SetModel( oModel )

//Adiciona no nosso View um controle do tipo FormFields(antiga enchoice)
oView:AddField( 'VIEW_ZA1' , oStruZA1, 'ZA1MASTER'   )

//Adiciona no nosso View um controle do tipo FormGrid(antiga newgetdados)
oView:AddGrid(  'VIEW_ZA2' , oStruZA2, 'ZA2DETAIL'   )

//Adiciona no nosso View um controle do tipo FormGrid(antiga newgetdados)
oView:AddField( 'VIEW_CALC', oCalc1  , 'COMP022CALC1' )

// Cria Folder na view
oView:CreateFolder( 'PASTAS' )
// Cria pastas nas folders
oView:AddSheet( 'PASTAS', 'ABA01', 'Cabealho' )
oView:AddSheet( 'PASTAS', 'ABA02', 'Item'      )

// Criar "box" horizontal para receber algum elemento da view
oView:CreateHorizontalBox( 'GERAL'  , 100,,, 'PASTAS', 'ABA01' )
oView:CreateHorizontalBox( 'CORPO'  ,  80,,, 'PASTAS', 'ABA02' )
oView:CreateHorizontalBox( 'RODAPE1',  10,,, 'PASTAS', 'ABA02' )
oView:CreateHorizontalBox( 'RODAPE2',  10,,, 'PASTAS', 'ABA02' )

// Relaciona o ID da View com o "box" para exibicao
oView:SetOwnerView( 'VIEW_ZA1' , 'GERAL'  )
oView:SetOwnerView( 'VIEW_ZA2' , 'CORPO'  )
oView:SetOwnerView( 'VIEW_CALC', 'RODAPE1' )

// Define campos que terao Auto Incremento
oView:AddIncrementField( 'VIEW_ZA2', 'ZA2_ITEM' )


Return oView


//-------------------------------------------------------------------
Static Function COMP022CAL( oFW, lPar )
Local lRet := .T.

If lPar
	lRet := ( Mod( Val( oFw:GetValue( 'ZA2DETAIL', 'ZA2_AUTOR' ) ) , 2 ) == 0 )
Else
	lRet := ( Mod( Val( oFw:GetValue( 'ZA2DETAIL', 'ZA2_AUTOR' ) ) , 2 ) <> 0 )
EndIf
Return lRet

//-------------------------------------------------------------------
Static Function COMP022CM1( oModel )
Local aArea      := GetArea()
Local aAreaZA0   := ZA0->( GetArea() )
Local nOperation := oModel:GetOperation()
Local nI         := 0

If     nOperation == MODEL_OPERATION_INSERT

	oModelZA2 := oModel:GetModel( 'ZA2DETAIL' )

	For nI := 1 To oModelZA2:GetQtdLine()

		ZA0->( dbSetOrder( 1 ) )

		If ZA0->( dbSeek( xFilial( 'ZA0' ) + FwFldGet( 'ZA2_AUTOR', nI ) ) )
			RecLock( 'ZA0', .F. )
			ZA0->ZA0_QTDMUS++
			MsUnlock()
		EndIf

	Next

ElseIf nOperation == MODEL_OPERATION_DELETE

	oModelZA2 := oModel:GetModel( 'ZA2DETAIL' )

	For nI := 1 To oModelZA2:GetQtdLine()

		ZA0->( dbSetOrder( 1 ) )

		If ZA0->( dbSeek( xFilial( 'ZA0' ) + FwFldGet( 'ZA2_AUTOR', nI ) ) )
			RecLock( 'ZA0', .F. )
			ZA0->ZA0_QTDMUS := Max( 0, ZA0->ZA0_QTDMUS - 1 )
			MsUnlock()
		EndIf

	Next

EndIf

FWFormCommit( oModel )

RestArea( aAreaZA0 )
RestArea( aArea )
Return NIL



//-------------------------------------------------------------------
Static Function COMP022CM2( oModel )
Local aArea      := GetArea()
Local aAreaZA0   := ZA0->( GetArea() )
Local nOperation := oModel:GetOperation()
Local nI         := 0
Local lOk        := .T.
Local aSaveLines := FWSaveRows()

If         nOperation == MODEL_OPERATION_INSERT
	nVal := 1
ElseIf     nOperation == MODEL_OPERATION_UPDATE
	nVal := 0
ElseIf     nOperation == MODEL_OPERATION_DELETE
	nVal := -1
EndIf

oModelZA0 := FWLoadModel( 'COMP011_MVC' )
oModelZA0:SetOperation( MODEL_OPERATION_UPDATE )

oModelZA2 := oModel:GetModel( 'ZA2DETAIL' )

For nI := 1 To oModelZA2:GetQtdLine()

	oModelZA2:GoLine( nI )

	ZA0->( dbSetOrder( 1 ) )

	If ZA0->( dbSeek( xFilial( 'ZA0' ) + FwFldGet( 'ZA2_AUTOR', nI, oModel ) ) )

		oModelZA0:Activate()

		If      nOperation == MODEL_OPERATION_INSERT

			If !oModelZA0:SetValue( 'ZA0MASTER', 'ZA0_QTDMUS', oModelZA0:GetValue( 'ZA0MASTER', 'ZA0_QTDMUS' ) + 1 ) // FwFldGet( 'ZA0_QTDMUS' ,, oModelZA0 )
				lOk := .F.
				Help( ,, 'Help',, 'Erro ao gravar quantidade', 1, 0 )
			EndIf

		ElseIf  nOperation == MODEL_OPERATION_DELETE

			If !oModelZA0:SetValue( 'ZA0MASTER', 'ZA0_QTDMUS', Max( 0, oModelZA0:GetValue( 'ZA0MASTER', 'ZA0_QTDMUS' ) - 1 ) )  // FwFldGet( 'ZA0_QTDMUS' ,, oModelZA0 )
				lOk := .F.
				Help( ,, 'Help',, 'Erro ao gravar quantidade', 1, 0 )
			EndIf

		ElseIf  nOperation == MODEL_OPERATION_UPDATE

			If      oModelZA2:IsDeleted()
				If !oModelZA0:SetValue( 'ZA0MASTER', 'ZA0_QTDMUS', Max( 0, oModelZA0:GetValue( 'ZA0MASTER', 'ZA0_QTDMUS' ) - 1 ) ) // FwFldGet( 'ZA0_QTDMUS' ,, oModelZA0 )
					lOk := .F.
					Help( ,, 'Help',, 'Erro ao gravar quantidade', 1, 0 )
				EndIf

			ElseIf 	oModelZA2:IsUpDated()

			EndIf

		EndIf

		If lOk
			If ( lOk  := oModelZA0:VldData() )
				oModelZA0:CommitData()
			Else
				Help( ,, 'Help',, 'Erro ao gravar quantidade', 1, 0 )
				Exit
			EndIf

		Else
			Exit

		EndIf

		oModelZA0:DeActivate()

	EndIf

Next

FWFormCommit( oModel )

FWRestRows( aSaveLines )

RestArea( aAreaZA0 )
RestArea( aArea )

Return NIL

